/* ------------------------------------------------------------------------*/
/* COPYRIGHT (c) 1992,1995 CONTEMPORARY CONTROL SYSTEMS, INC.              */
/* ------------------------------------------------------------------------*/
/* Project:       20020 DRIVER                                             */
/* Filename:      write20.c                                                */
/* Description:   simple write routine to test/show calls                  */
/* Version:       2.20                                                     */
/* Author:        Mike Justice                                             */
/* History:       ORIGINAL 09/14/92                                        */
/* ------------------------------------------------------------------------*/
#include "arcdef.h"

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

/* ------------------------------------------------------------------------- */
/* external variable definitions                                             */
/* ------------------------------------------------------------------------- */
extern USIGN16 arc20_port;
extern USIGN16 arc20_swport;
extern USIGN8  arc20_intr;
extern USIGN8  arc20_level;


/* ------------------------------------------------------------------------- */
/* global variable definition                                                */
/* ------------------------------------------------------------------------- */
USIGN8 buf[512];
USIGN8 userbuf[512];

/* ------------------------------------------------------------------------- */
/* function prototypes */
/* ------------------------------------------------------------------------- */
#include "arc20.h"


/****************************************************************************
 * FUNCTION NAME: main
 *
 * DESCRIPTION  : simple write routine
 *
 * RETURN VALUE : status
 ***************************************************************************/
void main(int argc, char *argv[])
{
    USIGN8 status,crhit,eschit,c;
    USIGN8 destid,index;
    char *s,cc;
    int i,ii;

    /* signon message */
    printf("Contemporary Control Systems, Inc.\n");
    printf("ARC20 Driver Write Test Routine V2.20, Copyright (c) 1993-1996\n");

    /* check arguments */
    if (argc > 1) {
        while (--argc > 0 && (*++argv)[0] == '-') {
            for (s = argv[0] + 1; *s != '\0'; s++) {
                switch (*s) {
                /* port address */
                case 'p':
                case 'P':
                    i = 1;
                    arc20_port = 0;
                    cc = *(s+1);
                    while (cc != '\0') {
                        if ((cc >= '0') && (cc <= '9')) {
                            arc20_port = arc20_port * 16 + (cc - 0x30);
                        }
                        else if ((cc >= 'a') && (cc <= 'f')) {
                            arc20_port = arc20_port * 16 + (cc - 0x57);
                        }
                        else if ((cc >= 'A') && (cc <= 'F')) {
                            arc20_port = arc20_port * 16 + (cc - 0x37);
                        }
                        i++;
                        cc = *(s+i);
                    }
                    break;
                
                /* interrupt enable */
                case 'i':
                case 'I':
                    arc20_intr = FALSE;
                    if (*(s+1) == '1') arc20_intr = TRUE;
                    break;
                /* interrupt level */
                case 'l':
                case 'L':
                    arc20_level = 0;
                    if ((*(s+1) >= '0') && (*(s+1) <= '9'))
                        arc20_level = *(s+1) - (USIGN8)0x30;
                    break;
                /* switch address */
                case 's':
                case 'S':
                    i = 1;
                    arc20_swport = 0;
                    cc = *(s+1);
                    while (cc != '\0') {
                        if ((cc >= '0') && (cc <= '9')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x30);
                        }
                        else if ((cc >= 'a') && (cc <= 'f')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x57);
                        }
                        else if ((cc >= 'A') && (cc <= 'F')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x37);
                        }
                        i++;
                        cc = *(s+i);
                    }
                    break;
                case 'h':
                case 'H':
                case '?':
                    printf("Options: -p<port address> -s<switch address> -i<interrupt> -l<level>\n");
                    printf("         port address default = 2e0\n");
                    printf("         switch address default = 2e9\n");
                    printf("         interrupt 0=off,1=on\n");
                    printf("         level = 0-7\n");
                    exit(0);
                    break;
                default:
                    break;
                }
            }
        }
    }

    /* initialize the driver */
    status = arc20_init();
    if (status == 0) {
        printf("Initialization Failure\n");
        exit(0);
    }
    else {
        printf("Node ID = %d (%XH) Initialized\n\n",status,status);
        printf("Enter Destination ID (Hex): ");
        destid = 0;
        crhit = FALSE;
        while (crhit == FALSE) {
            if (kbhit()) {
                c = (USIGN8)getche();
                if ((c >= '0') && (c <= '9')) {
                    destid <<= 4;
                    destid |= (c - 0x30);
                }
                else {
                    if (c == 0xd) crhit = TRUE;
                    else {
                        if ((c >= 'a') && (c <= 'f')) c -= 0x20;
                        if ((c >= 'A') && (c <= 'F')) {
                            destid <<= 4;
                            destid |= (c - 0x37);
                        }
                    }
                }
            }
        }
        printf("\nEnter Packet followed by Enter, or Hit ESC key to Exit\n");
    }

printf("Destid = %X\n",destid);

    /* read a packet, printf contents, if keyboard hit exit */
    eschit = FALSE;
    crhit = FALSE;
    index = 0;
    while (eschit == FALSE) {

        /* get packet characters */
        /* check keyboard */
        if(kbhit()) {
            c = (USIGN8)getch();
            if (c == 0x1b) eschit = TRUE;
            else {
                if (c == 0xd) {
                    crhit = TRUE;
                    userbuf[index] = 0;
                }
                else {
                    printf("%c",c);
                    userbuf[index] = c;
                    index++;
                }
            }
        }

        if (crhit == TRUE) {

            /* fill the packet with info SID, DID, msbcount, lsbcount, data,... */
            buf[0] = destid;
            buf[1] = destid; 
            buf[2] = 0;
            buf[3] = index;
            for (ii=0; ii<index; ii++) buf[4+ii] = userbuf[ii];

            /* any packets for me */
            status = arc20_write(&buf[0]);

            /* yes, then print */
            if (status == E_TX_BUSY) printf("\nTransmitter Busy \n");
            printf("\nEnter Packet followed by Enter, or Hit ESC key to Exit\n");
            crhit = FALSE;
            index = 0;
        }
    }

    /* exit to dos */
    arc20_exit();
    printf("Exiting to DOS\n");
    exit(0);
}

